@@ -1,11 +1,13 @@
 // termmouse.c (part of mintty)
-// Copyright 2008-12 Andy Koppe, 2017 Thomas Wolff
+// Copyright 2008-12 Andy Koppe, 2017-20 Thomas Wolff
 // Based on code from PuTTY-0.60 by Simon Tatham and team.
 // Licensed under the terms of the GNU General Public License v3 or later.
 
 #include "termpriv.h"
 #include "win.h"
 #include "child.h"
+#include "charset.h"  // cs__utftowcs
+#include "tek.h"
 
 /*
  * Fetch the character at a particular position in a line array.
@@ -298,25 +300,38 @@ send_mouse_event(mouse_action a, mouse_button b, mod_keys mods, pos p)
 
   uint x = p.x + 1, y = p.y + 1;
 
-  switch (b) {
-    when MBT_4:
-      b = MBT_LEFT; mods |= MDK_ALT;
-    when MBT_5:
-      b = MBT_RIGHT; mods |= MDK_ALT;
-    otherwise:;
+  if (a != MA_WHEEL) {
+    if (cfg.old_xbuttons)
+      switch (b) {
+        when MBT_4:
+          b = MBT_LEFT; mods |= MDK_ALT;
+        when MBT_5:
+          b = MBT_RIGHT; mods |= MDK_ALT;
+        otherwise:;
+      }
+    else
+      switch (b) {
+        when MBT_4:
+          b = 129;
+        when MBT_5:
+          b = 130;
+        otherwise:;
+      }
   }
 
   uint code = b ? b - 1 : 0x3;
 
   if (a != MA_RELEASE)
     code |= a * 0x20;
-  else if (term.mouse_enc != ME_XTERM_CSI)
+  else if (term.mouse_enc != ME_XTERM_CSI && term.mouse_enc != ME_PIXEL_CSI)
     code = 0x3;
 
   code |= (mods & ~cfg.click_target_mod) * 0x4;
 
   if (term.mouse_enc == ME_XTERM_CSI)
     child_printf("\e[<%u;%u;%u%c", code, x, y, (a == MA_RELEASE ? 'm' : 'M'));
+  else if (term.mouse_enc == ME_PIXEL_CSI)
+    child_printf("\e[<%u;%u;%u%c", code, p.pix + 1, p.piy + 1, (a == MA_RELEASE ? 'm' : 'M'));
   else if (term.mouse_enc == ME_URXVT_CSI)
     child_printf("\e[%u;%u;%uM", code + 0x20, x, y);
   else {
@@ -355,6 +370,7 @@ box_pos(pos p)
 {
   p.y = min(max(0, p.y), term.rows - 1);
   p.x = min(max(0, p.x), term.cols - 1);
+  // p.piy and p.pix already clipped in translate_pos()
   return p;
 }
 
@@ -363,6 +379,14 @@ get_selpoint(const pos p)
 {
   pos sp = { .y = p.y + term.disptop, .x = p.x, .r = p.r };
   termline *line = fetch_line(sp.y);
+
+  // Adjust to presentational direction.
+  if (line->lattr & LATTR_PRESRTL) {
+    sp.x = term.cols - 1 - sp.x;
+    sp.r = !sp.r;
+  }
+
+  // Adjust to double-width line display.
   if ((line->lattr & LATTR_MODE) != LATTR_NORM)
     sp.x /= 2;
 
@@ -370,8 +394,12 @@ get_selpoint(const pos p)
   * Transform x through the bidi algorithm to find the _logical_
   * click point from the physical one.
   */
-  if (term_bidi_line(line, p.y) != null)
+  if (term_bidi_line(line, p.y) != null) {
+#ifdef debug_bidi_cache
+    printf("mouse @ log %d -> vis %d\n", sp.x, term.post_bidi_cache[p.y].backward[sp.x]);
+#endif
     sp.x = term.post_bidi_cache[p.y].backward[sp.x];
+  }
 
   // Back to previous cell if current one is second half of a wide char
   if (line->chars[sp.x].chr == UCSWIDE)
@@ -394,7 +422,7 @@ send_keys(char *code, uint len, uint count)
 }
 
 static bool
-is_app_mouse(mod_keys *mods_p)
+check_app_mouse(mod_keys *mods_p)
 {
   if (term.locator_1_enabled)
     return true;
@@ -405,19 +433,37 @@ is_app_mouse(mod_keys *mods_p)
   return cfg.clicks_target_app ^ override;
 }
 
-void
+bool
 term_mouse_click(mouse_button b, mod_keys mods, pos p, int count)
 {
+  compose_clear();
+
   if (term.hovering) {
     term.hovering = false;
     win_update(true);
   }
 
-  if (is_app_mouse(&mods)) {
+  bool res = true;
+  if (tek_mode == TEKMODE_GIN) {
+    char c = '`';
+    switch (b) {
+      when MBT_LEFT: c = 'l';
+      when MBT_MIDDLE: c = 'm';
+      when MBT_RIGHT: c = 'r';
+      when MBT_4: c = 'p';
+      when MBT_5: c = 'q';
+    }
+    if (mods & MDK_SHIFT)
+      c ^= ' ';
+    c |= 0x80;
+    child_send(&c, 1);
+    tek_send_address();
+  }
+  else if (check_app_mouse(&mods)) {
     if (term.mouse_mode == MM_X10)
       mods = 0;
     send_mouse_event(MA_CLICK, b, mods, box_pos(p));
-    term.mouse_state = b;
+    term.mouse_state = (int)b;
   }
   else {
     // generic transformation M4/M5 -> Alt+left/right;
@@ -443,23 +489,31 @@ term_mouse_click(mouse_button b, mod_keys mods, pos p, int count)
       // trying to ignore WM_CAPTURECHANGED does not help
       if (!alt || fake_alt)
         win_popup_menu(mods);
+      else
+        res = false;
     }
     else if (b == MBT_MIDDLE && (mods & ~MDK_SHIFT) == MDK_CTRL) {
       if (cfg.zoom_mouse)
         win_zoom_font(0, mods & MDK_SHIFT);
+      else
+        res = false;
     }
     else if ((b == MBT_RIGHT && rca == RC_PASTE) ||
-             (b == MBT_MIDDLE && mca == MC_PASTE)) {
+             (b == MBT_MIDDLE && mca == MC_PASTE))
+    {
       if (!alt)
         term.mouse_state = shift_or_ctrl ? MS_COPYING : MS_PASTING;
+      else
+        res = false;
     }
     else if ((b == MBT_RIGHT && rca == RC_ENTER) ||
              (b == MBT_MIDDLE && mca == MC_ENTER)) {
       child_send("\r", 1);
     }
-    else if (b == MBT_LEFT && mods == MDK_SHIFT && rca == RC_EXTEND)
+    else if (b == MBT_LEFT && mods == MDK_SHIFT && rca == RC_EXTEND) {
       term.mouse_state = MS_PASTING;
-    else if (b == MBT_LEFT && mods == MDK_CTRL) {
+    }
+    else if (b == MBT_LEFT && (mods & ~cfg.click_target_mod) == MDK_CTRL) {
       if (count == cfg.opening_clicks) {
         // Open word under cursor
         p = get_selpoint(box_pos(p));
@@ -470,10 +524,13 @@ term_mouse_click(mouse_button b, mod_keys mods, pos p, int count)
         sel_spread();
         win_update(true);
       }
+      else
+        res = false;
     }
     else if (b == MBT_MIDDLE && mca == MC_VOID) {
+      // res = true; // MC_VOID explicitly ignores the click
     }
-    else {
+    else if ((mods & (MDK_CTRL | MDK_ALT)) != (MDK_CTRL | MDK_ALT)) {
       // Only clicks for selecting and extending should get here.
       p = get_selpoint(box_pos(p));
       term.mouse_state = -count;
@@ -494,22 +551,37 @@ term_mouse_click(mouse_button b, mod_keys mods, pos p, int count)
       win_capture_mouse();
       win_update(true);
     }
+    else {
+      res = false;
+    }
   }
+
+  return res;
 }
 
 void
 term_mouse_release(mouse_button b, mod_keys mods, pos p)
 {
+  compose_clear();
+
   int state = term.mouse_state;
   term.mouse_state = 0;
   switch (state) {
     when MS_COPYING: term_copy();
     when MS_PASTING: win_paste();
-    when MS_OPENING:
-      term_open();
+    when MS_OPENING: {
+      termline *line = fetch_line(p.y + term.disptop);
+      int urli = line->chars[p.x].attr.link;
+      release_line(line);
+      char * url = geturl(urli);
+      if (url)
+        win_open(cs__utftowcs(url), true);  // win_open frees its argument
+      else
+        term_open();
       term.selected = false;
       term.hovering = false;
       win_update(true);
+    }
     when MS_SEL_CHAR or MS_SEL_WORD or MS_SEL_LINE: {
       // Finish selection.
       if (term.selected && cfg.copy_on_select)
@@ -572,7 +644,7 @@ term_mouse_release(mouse_button b, mod_keys mods, pos p)
       last_dest = dest;
     }
     otherwise:
-      if (is_app_mouse(&mods)) {
+      if (check_app_mouse(&mods)) {
         if (term.mouse_mode >= MM_VT200)
           send_mouse_event(MA_RELEASE, b, mods, box_pos(p));
       }
@@ -593,6 +665,8 @@ sel_scroll_cb(void)
 void
 term_mouse_move(mod_keys mods, pos p)
 {
+  compose_clear();
+
   //printf("mouse_move %d+%d/2\n", p.x, p.r);
   pos bp = box_pos(p);
 
@@ -616,36 +690,49 @@ term_mouse_move(mod_keys mods, pos p)
     win_update(true);
   }
   else if (term.mouse_state == MS_OPENING) {
+    // let's not clear link opening state when just moving the mouse (#1039)
+    // but only after hovering out of the link area (below)
+#ifdef link_opening_only_if_unmoved
     term.mouse_state = 0;
     term.selected = false;
     win_update(true);
+#endif
   }
   else if (term.mouse_state > 0) {
     if (term.mouse_mode >= MM_BTN_EVENT)
-      send_mouse_event(MA_MOVE, term.mouse_state, mods, bp);
+      send_mouse_event(MA_MOVE, (mouse_button)term.mouse_state, mods, bp);
   }
   else {
     if (term.mouse_mode == MM_ANY_EVENT)
       send_mouse_event(MA_MOVE, 0, mods, bp);
   }
 
-  if (mods == MDK_CTRL && term.has_focus) {
+  if (!check_app_mouse(&mods) && (mods & ~cfg.click_target_mod) == MDK_CTRL && term.has_focus) {
     p = get_selpoint(box_pos(p));
     term.hover_start = term.hover_end = p;
     if (!hover_spread_empty()) {
       term.hovering = true;
+      termline *line = fetch_line(p.y);
+      term.hoverlink = line->chars[p.x].attr.link;
+      release_line(line);
       win_update(true);
     }
     else if (term.hovering) {
       term.hovering = false;
       win_update(true);
     }
+    //printf("->hovering %d (opening %d)\n", term.hovering, term.mouse_state == MS_OPENING);
+    // clear link opening state after hovering out of link area
+    if (!term.hovering && term.mouse_state == MS_OPENING)
+      term.mouse_state = 0;
   }
 }
 
 void
-term_mouse_wheel(int delta, int lines_per_notch, mod_keys mods, pos p)
+term_mouse_wheel(bool horizontal, int delta, int lines_per_notch, mod_keys mods, pos p)
 {
+  compose_clear();
+
   if (term.hovering) {
     term.hovering = false;
     win_update(true);
@@ -653,10 +740,17 @@ term_mouse_wheel(int delta, int lines_per_notch, mod_keys mods, pos p)
 
   enum { NOTCH_DELTA = 120 };
 
-  static int accu;
+  static int accu = 0;
   accu += delta;
 
-  if (is_app_mouse(&mods)) {
+  if (tek_mode == TEKMODE_GIN) {
+    int step = (mods & MDK_SHIFT) ? 40 : (mods & MDK_CTRL) ? 1 : 4;
+    if (horizontal ^ (mods & MDK_CTRL))
+      tek_move_by(0, step * delta / NOTCH_DELTA);
+    else
+      tek_move_by(step * delta / NOTCH_DELTA, 0);
+  }
+  else if (check_app_mouse(&mods)) {
     if (strstr(cfg.suppress_wheel, "report"))
       return;
     // Send as mouse events, with one event per notch.
@@ -664,11 +758,17 @@ term_mouse_wheel(int delta, int lines_per_notch, mod_keys mods, pos p)
     if (notches) {
       accu -= NOTCH_DELTA * notches;
       mouse_button b = (notches < 0) + 1;
+      if (horizontal)
+        b = 5 - b;
       notches = abs(notches);
-      do send_mouse_event(MA_WHEEL, b, mods, p); while (--notches);
+      do
+        send_mouse_event(MA_WHEEL, b, mods, box_pos(p));
+      while (--notches);
     }
   }
-  else if ((mods & ~MDK_SHIFT) == MDK_CTRL) {
+  else if (horizontal) {
+  }
+  else if (cfg.zoom_mouse && (mods & ~MDK_SHIFT) == MDK_CTRL) {
     if (strstr(cfg.suppress_wheel, "zoom"))
       return;
     if (cfg.zoom_mouse) {
@@ -679,32 +779,41 @@ term_mouse_wheel(int delta, int lines_per_notch, mod_keys mods, pos p)
       }
     }
   }
-  else if (!(mods & ~MDK_SHIFT)) {
+  else if (!(mods & ~(MDK_SHIFT | MDK_CTRL | MDK_ALT))) {
+    if (mods & MDK_CTRL)
+      lines_per_notch = 1;
+    else if (cfg.lines_per_notch > 0)
+      lines_per_notch = min(cfg.lines_per_notch, term.rows - 1);
+
     // Scroll, taking the lines_per_notch setting into account.
     // Scroll by a page per notch if setting is -1 or Shift is pressed.
-    int lines_per_page = max(1, term.rows - 1);
+    int lines_per_page = max(1, term.rows);
     if (lines_per_notch == -1 || mods & MDK_SHIFT)
       lines_per_notch = lines_per_page;
     int lines = lines_per_notch * accu / NOTCH_DELTA;
+    //printf("mouse lines %d per notch %d accu %d\n", lines, lines_per_notch, accu);
     if (lines) {
       accu -= lines * NOTCH_DELTA / lines_per_notch;
-      if (!term.on_alt_screen || term.show_other_screen) {
+      if ((!term.on_alt_screen || term.show_other_screen) && !(mods & MDK_ALT)) {
         if (strstr(cfg.suppress_wheel, "scrollwin"))
           return;
         term_scroll(0, -lines);
       }
-      else if (term.wheel_reporting) {
-        if (strstr(cfg.suppress_wheel, "scrollapp"))
+      else if (term.wheel_reporting || term.wheel_reporting_xterm) {
+        if (strstr(cfg.suppress_wheel, "scrollapp") && !term.wheel_reporting_xterm)
           return;
         // Send scroll distance as CSI a/b events
         bool up = lines > 0;
         lines = abs(lines);
         int pages = lines / lines_per_page;
         lines -= pages * lines_per_page;
-        if (term.app_wheel) {
+        if (term.app_wheel && !term.wheel_reporting_xterm) {
           send_keys(up ? "\e[1;2a" : "\e[1;2b", 6, pages);
           send_keys(up ? "\eOa" : "\eOb", 3, lines);
         }
+        else if (term.vt52_mode) {
+          send_keys(up ? "\eA" : "\eB", 2, lines);
+        }
         else {
           send_keys(up ? "\e[5~" : "\e[6~", 4, pages);
           char code[3] =
